#!/bin/bash
function pause()
{
  #echo "Press any key to quit:"
  #read -n1 -s key
  exit 1
}

function usage {
echo Usage:
echo "$0 [-u <user:password>]"
echo "Compress firmware image file with gzip. "
echo " -r repository to use for uploads [https://nexus.buzztime.com/repository/firmware]"
echo "if -u is specified, use curl --upload-file to upload"
echo "      zipped image "
echo "      manifest.json"
echo " to nexus.buzztime.com/repository/firmware/"
}

NEXUS_REPO="https://nexus.buzztime.com/repository/firmware"
NEXUS_USER=""
while getopts ":hu:r:" opt; do
    case $opt in
    	h) usage
    	exit 0
    	;;
      r) NEXUS_REPO="${OPTARG%%/}"
	#remove the trailing slash if present
      ;;
      u) NEXUS_USER="$OPTARG"
      ;;
    esac
done

echo "setting build environment"
set -e
. build/envsetup.sh >/dev/null && setpaths

export PATH=$ANDROID_BUILD_PATHS:$PATH
TARGET_PRODUCT=`get_build_var TARGET_PRODUCT`
TARGET_HARDWARE=`get_build_var TARGET_BOARD_HARDWARE`
TARGET_DEVICE=`get_build_var TARGET_DEVICE`
TARGET_BUILD_VARIANT=`get_build_var TARGET_BUILD_VARIANT`

#Used to gather build information from files at root, building the BUILD_TAG
if [ -f  ${TARGET_PRODUCT}_build.id ]; then 
  BUILD_ID=`cat ${TARGET_PRODUCT}_build.id`
else
  BUILD_ID="0.1.0"
fi
if [ -f  ${TARGET_PRODUCT}_build_number.id ]; then 
  BUILD_NUMBER=`cat ${TARGET_PRODUCT}_build_number.id`
else
  BUILD_NUMBER="0000"
fi
BUILD_TAG="${BUILD_ID}-${BUILD_NUMBER}-${TARGET_BUILD_VARIANT}"

# used when we thought we could deliver updates in addition to full images
#OTA_PATH=../ota-$TARGET_PRODUCT/$BUILD_ID-$BUILD_NUMBER
IMAGE_PATH=rockdev/Image-$TARGET_PRODUCT

echo TARGET_PRODUCT=$TARGET_PRODUCT
echo BUILD_TAG=$BUILD_TAG
echo IMAGE_PATH=$IMAGE_PATH
echo TARGET_DEVICE=$TARGET_DEVICE

echo "publish $TARGET_PRODUCT-rkimage-$BUILD_TAG.img ..."
cd rockdev

rkImage="$TARGET_PRODUCT-rkimage-$BUILD_TAG.img"
rkManifest="$TARGET_PRODUCT-rkimage-$BUILD_TAG.manifest.json"
echo "rkimage: ${rkImage}"
echo "rkManifest: ${rkManifest}"

imgMd5="$(openssl dgst -md5 -r "${rkImage}")"
imgSha256="$(openssl dgst -sha256 -r "${rkImage}")"

echo "SHACALC: ${imgSha256%% *} : "
echo "MD5: ${imgMd5%% *} : "

if [ -f "${rkImage}.gzip" ]
then
   rm ${rkImage}.gzip
fi
if [ -f "${rkManifest}" ]
then
   rm ${rkManifest}
fi
echo "gzip : ${rkImage}"
gzip -S .gzip -k "${rkImage}"
echo "gzipped : ${rkImage}"
totalsize="$(du -b "${rkImage}" | cut -f1)"
GZipSize="$(du -b "${rkImage}.gzip" | cut -f1)"
imgGZipMd5="$(openssl dgst -md5 -r "${rkImage}.gzip")"
imgGZipSha256="$(openssl dgst -sha256 -r "${rkImage}.gzip")"
echo "building manifest: ${rkManifest}"

    echo -ne '{\n' > "${rkManifest}"
    echo -ne "  \"board.hardware\": \"$(get_build_var TARGET_BOARD_HARDWARE)\",\n" >> "${rkManifest}"
    echo -ne "  \"board.platform\": \"$(get_build_var TARGET_BOARD_PLATFORM)\",\n" >> "${rkManifest}"
    echo -ne "  \"build.id\": \"${BUILD_ID}\",\n" >> "${rkManifest}"
    echo -ne "  \"build.number\": \"${BUILD_NUMBER}\",\n" >> "${rkManifest}"
    echo -ne "  \"build.product\": \"$(get_build_var TARGET_PRODUCT)\",\n" >> "${rkManifest}"
    echo -ne "  \"build.device\": \"$(get_build_var TARGET_DEVICE)\",\n" >> "${rkManifest}"
    echo -ne "  \"build.model\": \"$(get_build_var PRODUCT_MODEL)\",\n" >> "${rkManifest}"
    echo -ne "  \"build.variant\": \"$(get_build_var TARGET_BUILD_VARIANT)\",\n" >> "${rkManifest}"
    echo -ne "  \"build.tag\": \"${BUILD_TAG}\",\n" >> "${rkManifest}"
    echo -ne "  \"gzip.name\": \"${rkImage}.gzip\",\n" >> "${rkManifest}"
    echo -ne "  \"gzip.sha256\":\"${imgGZipSha256%% *}\",\n" >> "${rkManifest}"
    echo -ne "  \"gzip.md5\":\"${imgGZipMd5%% *}\",\n" >> "${rkManifest}"
    echo -ne "  \"gzip.size\": ${GZipSize},\n" >> "${rkManifest}"
    echo -ne "  \"img.sha256\":\"${imgSha256%% *}\",\n" >> "${rkManifest}"
    echo -ne "  \"img.md5\":\"${imgMd5%% *}\",\n" >> "${rkManifest}"
    echo -ne "  \"img.size\": ${totalsize},\n" >> "${rkManifest}"
    echo -ne "  \"img.name\": \"${rkImage}\",\n" >> "${rkManifest}"
    echo -ne "  \"url\": \"${NEXUS_REPO}/${rkImage}.gzip\"\n" >> "${rkManifest}"
   echo -ne "  }\n" >> "${rkManifest}"

echo "====== Manifest Info ========"
cat ${rkManifest}

if [ ! -z "${NEXUS_USER}" ]
then
  curl -vvv --user "${NEXUS_USER}" --upload-file "${rkImage}.gzip" "${NEXUS_REPO}/${rkImage}.gzip"
  curl -vvv --user "${NEXUS_USER}" --upload-file "${rkManifest}" "${NEXUS_REPO}/${rkManifest}"

fi

#echo "Press any key to quit:"
#read -n1 -s key
croot
exit 0
