package com.buzztime.tabletprovisioner.model;

import com.android.ddmlib.IDevice;

import javax.swing.*;
import java.util.EnumSet;

/**
 * Info about a specific tablet,
 * Assumes that the IDevice may coe and go, so this maps off the device name
 * Created by mstevens on 5/2/16.
 */
public class DeviceInfo {
    public enum TabletModel{
        TAB2 ("GT-P3113"),
        TAB3 ("SM-T210R"),
        TAB4 ("SM-T230NU"),
        HALO_FLO ("Halo on Flo"),
        HALO_SLA ("HALO"),
        HALO ("BZT-T101"),
        OTHER ("----");

        TabletModel(String model) {
            this.model = model;
        }

        String model;
    }
    public static EnumSet<TabletModel> HALO_MODELS = EnumSet.of(TabletModel.HALO, TabletModel.HALO_FLO, TabletModel.HALO_SLA);

    public enum Done {
        PROVISION_DATA_SET,
        ENCRYPT_TABLET,
        CONNECT_WIFI,
        VERIFY_HANDSHAKE,
        UPDATE_BTCORE,
        START_BTCORE,
        PUSH_PACKAGES,
    }


    public String name;
    public String serialno = null;
    public String model = null;
    public TabletModel tabletModel = TabletModel.OTHER;
    public boolean provisionStarted;
    public SwingWorker mWorker = null;


    DevicePackageInfo[] installedPackages;

    public DeviceInfo(IDevice device, String serialno) {
        this.name = device.getName();
        this.serialno = serialno;
        this.model = device.getProperty("ro.product.model");
        if (model!=null) {
            for (TabletModel m : TabletModel.values()) {
                if (model.equals( m.model)) {
                    tabletModel = m;
                    break;
                }
            }
        }

        // async assignment
        installedPackages = null;
    }


    public DevicePackageInfo[] getInstalledPackages() {
        return installedPackages;
    }

    public void setInstalledPackages(DevicePackageInfo[] installedPackages) {
        this.installedPackages = installedPackages;
    }
}
