package com.buzztime.tabletprovisioner.soap;

import com.buzztime.tabletprovisioner.model.ManifestPackageInfo;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlSerializer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by mstevens on 4/22/16.
 */
public interface IContentDeliveryService {
    String SOAP_ENV = "http://www.w3.org/2003/05/soap-envelope";
    String ADDRESSING = "http://www.w3.org/2005/08/addressing";
    String BZ_2011_10 = "http://services.buzztime.com/2011/10";
    String BZ_2011_04 = "http://services.buzztime.com/2011/04";
    String[]  soapEnvironmentUrl=
            {
                    "https://services.dev.buzztime.com/Operations/Network/ContentDeliveryService/ContentDeliveryService.svc",
                    "https://services.int.buzztime.com/Operations/Network/ContentDeliveryService/ContentDeliveryService.svc",
                    "https://services.stg.buzztime.com/Operations/Network/ContentDeliveryService/ContentDeliveryService.svc",
                    "https://services.buzztime.com/Operations/Network/ContentDeliveryService/ContentDeliveryService.svc"
            };


    enum SOAP {
        _Envelope,
            _Header,
                    _To,
                _Action,
                _MessageId,
                _ReplyTo,
                    _Address,
            _Body;

        public static boolean contains(String name){
            for (SOAP elt : values()){
                if (elt.name().equals(name))
                    return true;
            }
            return false;
        }
    }

    class GetTabletSoftwareManifest   {
        static final String ACTION = "http://services.buzztime.com/2011/04/IContentDeliveryService/GetTabletSoftwareManifest";
        static final String MESSAGE_ID = "urn:uuid:8003f209-e301-4b21-a042-cbeacbb167fb";
        private enum NODES {
            _GetTabletSoftwareManifestResponse ,
                _Manifest,
                    _firmwareInfo,
                        _nil,
                    _firmware,
                    _softwareInfo,
                        _releaseVersion,
                    _software,
                        _item,
                            _package,
                            _version,
                            _md5,
                            _url,
                            _filePath,
                            _buildNumber,
            }

        public String softwareReleaseVersion;
        public List<ManifestPackageInfo> mManifestPackageInfoList = new ArrayList<ManifestPackageInfo>();

        public static void createSoapRequest( XmlSerializer soap,  String url, String serialNumber ) throws IOException {
            soap.startDocument("UTF-8", true);
            soap.setPrefix("s", SOAP_ENV);
            soap.setPrefix("a", ADDRESSING);
            soap.startTag(SOAP_ENV, "Envelope");
            soap.startTag(SOAP_ENV, "Header");

            {
                soap.startTag(ADDRESSING, "To");
                soap.attribute(ADDRESSING, "mustUnderstand", "1");
                soap.text( url );
                soap.endTag(ADDRESSING, "To");

                soap.startTag(ADDRESSING, "Action");
                soap.attribute(ADDRESSING, "mustUnderstand", "1");
                soap.text(ACTION);
                soap.endTag(ADDRESSING, "Action");

                soap.startTag(ADDRESSING, "MessageId");
                soap.text(MESSAGE_ID);
                soap.endTag(ADDRESSING, "MessageId");

                soap.startTag(ADDRESSING, "ReplyTo");
                soap.startTag(ADDRESSING, "Address");
                soap.text("http://www.w3.org/2005/08/addressing/anonymous");
                soap.endTag(ADDRESSING, "Address");
                soap.endTag(ADDRESSING, "ReplyTo");
            }
            soap.endTag(SOAP_ENV, "Header");

            soap.startTag(SOAP_ENV, "Body");
            soap.startTag(null, "GetTabletSoftwareManifestRequest");
            soap.attribute(null, "xmlns", BZ_2011_10);
            soap.startTag(null, "SerialNumber");
            soap.attribute(null, "xmlns", BZ_2011_04);
            soap.text( serialNumber );
            soap.endTag(null, "SerialNumber");
            soap.endTag(null, "GetTabletSoftwareManifestRequest");
            soap.endTag(SOAP_ENV, "Body");
            soap.endDocument();
        }

        /**
         * parse the Soap response and build the manifest
         * @param xpp
         * @return
         * @throws XmlPullParserException
         * @throws IOException
         */
        public static GetTabletSoftwareManifest createFromSoap(XmlPullParser xpp)
                throws XmlPullParserException, IOException {

            GetTabletSoftwareManifest manifest = new GetTabletSoftwareManifest();
            ManifestPackageInfo packageInfo = null;

            for (int eventType = xpp.next(); eventType != XmlPullParser.END_DOCUMENT; eventType=xpp.next() ) {
                switch (eventType){
                    case XmlPullParser.START_DOCUMENT:
                        break;
                    case XmlPullParser.START_TAG: {
                        // filter out the soap envelope tags
                        if (SOAP.contains("_"+xpp.getName()))
                            break;
                        // have the enum do the conversion one time
                        // have to switch to uppercase because 'instance' is a java reserved word so it can't be
                        // the value of an enum
//                        System.out.println("XPP: " + xpp.getLineNumber() + xpp.getName());
                        switch (NODES.valueOf("_"+xpp.getName())){
                            case _GetTabletSoftwareManifestResponse:
                                break;
                            case _Manifest:
                                break;
                            case    _firmwareInfo:
                                break;
                            case        _nil:
                                boolean b = Boolean.parseBoolean( xpp.getAttributeValue(null, "nil"));
                                break;
                            case    _firmware:
                                break;
                            case    _softwareInfo:
                                break;
                            case        _releaseVersion:
                                manifest.softwareReleaseVersion = xpp.nextText();
                                break;
                            case    _software:
                                break;
                            case    _item:
                                // start a new package
                                packageInfo = new ManifestPackageInfo();
                                break;
                            case        _package:
                                packageInfo.packageName = xpp.nextText();
                                break;
                            case        _version:
                                packageInfo.versionName = xpp.nextText();
                                break;
                            case        _md5:
                                packageInfo.md5 = xpp.nextText();
                                break;
                            case        _url:
                                packageInfo.url = xpp.nextText();
                                break;
                            case        _filePath:
                                packageInfo.filePath = xpp.nextText();
                                packageInfo.parseInternalFilePath();

                                break;
                            case        _buildNumber:
                                packageInfo.versionCode = Integer.decode(xpp.nextText());
                                break;
                            default:
                        }
                    } break;

                    case XmlPullParser.END_TAG:
                        if (SOAP.contains("_"+xpp.getName()))
                            break;

                        switch (NODES.valueOf("_"+xpp.getName())){
                            case    _item:
                                // start a new package
                                manifest.mManifestPackageInfoList.add(packageInfo);
                                packageInfo = null;
                                break;

                            default:
                        }
                        break;
                    case XmlPullParser.END_DOCUMENT:
                        break;
                    case XmlPullParser.TEXT:
                        break;
                    default:
                }
            }
            return manifest;
        }

    }

}
