package com.buzztime.tabletprovisioner.soap;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;
import org.xmlpull.v1.XmlSerializer;

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * Created by mstevens on 4/22/16.
 */
public interface ISiteService {
    String TAG = "ISiteService";
    String ADDRESSING = "http://www.w3.org/2005/08/addressing";
    String INSTANCE = "http://www.w3.org/2001/XMLSchema-instance";
    String BZ_2011_01 = "http://services.buzztime.com/2011/01"; // data
    String BZ_2011_10 = "http://services.buzztime.com/2011/10";
    String BZ_2011_04 = "http://services.buzztime.com/2011/04";
    String BZ_2012_07 = "http://services.buzztime.com/2012/07"; //GetTabletConfiguration
    String BZ_GEOGRAPHY = "http://schemas.datacontract.org/2004/07/Buzztime.Core.Geography";
    String BZ_CUSTOMER = "http://schemas.datacontract.org/2004/07/Buzztime.Operations.Constituent.Customer";

    static final String[] EnvironmentUrl =
            {
                    "https://services.dev.buzztime.com/Operations/Constituent/Site/SiteService/SiteService.svc",
                    "https://services.int.buzztime.com/Operations/Constituent/Site/SiteService/SiteService.svc",
                    "https://services.stg.buzztime.com/Operations/Constituent/Site/SiteService/SiteService.svc",
                    "https://services.buzztime.com/Operations/Constituent/Site/SiteService/SiteService.svc"
            };


    class GetTabletConfiguration   {
        static final String ACTION = "http://services.buzztime.com/2011/01/ISiteService/GetTabletConfiguration";
        static final String MESSAGE_ID = "urn:uuid:a6058e8e-279a-4313-a210-58f8deb3d015";

        private enum NODES {
            _GetTabletConfigurationResponse ,
                _GMTOffset,
                _RouterSSID,
                _SiteChain,
                _HostIPAddress,
                _HostName,
                _IsValid,
                _RouterPassword,
                _SiteLocation,
                    _Address,
                        _City,
                        _Country,
                        _State,
                        _StreetAddress1,
                        _StreetAddress2,
                        _ZipCode,
                            _ZipCodeValue,
                    _Id,
                    _Name,  // used above too..
                _SitePhone,
                _TabletConfig,
                    _KeyValuePairOfstringstring,
                        _key,
                        _value,
                _SitePCIPAddress,
                _SpreadPort,
                _unknown;
            public static NODES getTag(String name ) {
                try{
                    return NODES.valueOf(name);
                } catch (IllegalArgumentException e){
                }
                return _unknown;
            }

        }

        String serialNumber;
        String deviceSerialNumber;
        String deviceIdentifier;

        SortedMap<String,String> map = new TreeMap<String,String>();

        String soapEnvelope;
        //<?xml version="1.0" encoding="UTF-8"?>
//    <s:Envelope xmlns:s="http://www.w3.org/2003/05/soap-envelope" xmlns:a="http://www.w3.org/2005/08/addressing">
//    <s:Header>
//    <a:To a:mustUnderstand="1">https://services.dev.buzztime.com/Operations/Constituent/Site/SiteService/SiteService.svc</a:To>
//    <a:Action a:mustUnderstand="1">http://services.buzztime.com/2011/01/ISiteService/GetTabletConfiguration</a:Action>
//    <a:MessageId>hurn:uuid:a6058e8e-279a-4313-a210-58f8deb3d015</a:MessageId>
//    <a:ReplyTo>
//    <a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>
//    </a:ReplyTo>
//    </s:Header>
//    <s:Body>
//    <GetTabletConfiguration xmlns="http://services.buzztime.com/2012/07">
//    <SerialNumber xmlns="http://services.buzztime.com/2011/01">R22c8008LAE</SerialNumber>
//    <DeviceIdentifier xmlns="http://services.buzztime.com/2011/01">R22c8008LAE</DeviceIdentifier>
//    <Security xmlns="http://services.buzztime.com/2011/01">1a6b12d88a5ffceb0c12e6951e8afbde</Security>
//    <FirmwareDisplay xmlns="http://services.buzztime.com/2011/01">halo_flo-userdebug 5.1.1 LMY48M eng.mstevens.20160415.113509 test-keys</FirmwareDisplay>
//    <FirmwareManufacturer xmlns="http://services.buzztime.com/2011/01">ASUS</FirmwareManufacturer>
//    <FirmwareModel xmlns="http://services.buzztime.com/2011/01">Halo on Flo</FirmwareModel>
//    <FirmwareOSVersion xmlns="http://services.buzztime.com/2011/01">5.1.1</FirmwareOSVersion>
//    <FirmwareSDKVersion xmlns="http://services.buzztime.com/2011/01">22</FirmwareSDKVersion>
//    <SoftwareVersion xmlns="http://services.buzztime.com/2011/01">---------</SoftwareVersion>
//    <DataFreeBytes xmlns="http://services.buzztime.com/2011/01">10640998400</DataFreeBytes>
//    <DataTotalBytes xmlns="http://services.buzztime.com/2011/01">13105287168</DataTotalBytes>
//    </GetTabletConfiguration>
//    </s:Body>
//    </s:Envelope>
        public static GetTabletConfiguration createSoapRequest( XmlSerializer soap, String url, String salt,
                              String serialNumber,
                              String deviceIdentifier,
                              String firmwareDisplay,
                              String firmwareManufacturer,
                              String firmwareModel,
                              String firmwareOSVersion,
                              int firmwareSDKVersion,
                              String softwareVersion,
                              long dataFreeBytes,
                              long dataTotalBytes) throws IOException {

            StringWriter envelope = null;
//            if (soap==null) {
//                soap = Xml.newSerializer();
//                envelope = new StringWriter();
//                soap.setOutput(envelope);
//            }

            soap.startDocument("UTF-8", true);
            soap.setPrefix("s", SOAP_ENV.NS);
            soap.setPrefix("a", ADDRESSING);
            soap.startTag(SOAP_ENV.NS, "Envelope");
            soap.startTag(SOAP_ENV.NS, "Header");

            {
                soap.startTag(ADDRESSING, "To");
                soap.attribute(ADDRESSING, "mustUnderstand", "1");
                soap.text(url);
                soap.endTag(ADDRESSING, "To");

                soap.startTag(ADDRESSING, "Action");
                soap.attribute(ADDRESSING, "mustUnderstand", "1");
                soap.text(ACTION);
                soap.endTag(ADDRESSING, "Action");

                soap.startTag(ADDRESSING, "MessageId");
                soap.text(MESSAGE_ID);
                soap.endTag(ADDRESSING, "MessageId");

                soap.startTag(ADDRESSING, "ReplyTo");
                soap.startTag(ADDRESSING, "Address");
                soap.text("http://www.w3.org/2005/08/addressing/anonymous");
                soap.endTag(ADDRESSING, "Address");
                soap.endTag(ADDRESSING, "ReplyTo");
            }
            soap.endTag(SOAP_ENV.NS, "Header");

            soap.startTag(SOAP_ENV.NS, "Body");
            {
                soap.startTag(null, "GetTabletConfigurationRequest");
                soap.attribute(null, "xmlns", BZ_2012_07);

                writeSoapTag(soap, "SerialNumber", BZ_2011_01, serialNumber);
                writeSoapTag(soap, "DeviceIdentifier", BZ_2011_01, deviceIdentifier);
                writeSoapTag(soap, "Security", BZ_2011_01, md5(serialNumber+deviceIdentifier+salt));
                writeSoapTag(soap, "FirmwareDisplay", BZ_2011_01, firmwareDisplay);
                writeSoapTag(soap, "FirmwareManufacturer", BZ_2011_01, firmwareManufacturer);
                writeSoapTag(soap, "FirmwareModel", BZ_2011_01, firmwareModel);
                writeSoapTag(soap, "FirmwareOSVersion", BZ_2011_01, firmwareOSVersion);
                writeSoapTag(soap, "FirmwareSDKVersion", BZ_2011_01, Integer.toString(firmwareSDKVersion));
                writeSoapTag(soap, "SoftwareVersion", BZ_2011_01, softwareVersion);
                writeSoapTag(soap, "DataFreeBytes", BZ_2011_01, Long.toString(dataFreeBytes));
                writeSoapTag(soap, "DataTotalBytes", BZ_2011_01, Long.toString(dataTotalBytes));

                soap.endTag(null, "GetTabletConfigurationRequest");
            }
            soap.endTag(SOAP_ENV.NS, "Body");
            soap.endDocument();

            GetTabletConfiguration request = new GetTabletConfiguration();
            if (envelope!=null)
                request.soapEnvelope = envelope.toString();

            return request;
        }


        private static void writeSoapTag(XmlSerializer soap,  String name, String xmlns, String value ) throws IOException{
            soap.startTag(null, name);
            soap.attribute(null, "xmlns", xmlns);
            soap.text(value);
            soap.endTag(null, name);
        }

        private static String md5(String str)
        {
            try
            {
                byte[] bytesOfMessage = str.getBytes("UTF-8");
                MessageDigest md = MessageDigest.getInstance("MD5");
                byte[] result = md.digest(bytesOfMessage);
                return String.format("%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
                        result[0], result[1],
                        result[2], result[3],
                        result[4], result[5],
                        result[6], result[7],
                        result[8], result[9],
                        result[10], result[11],
                        result[12], result[13],
                        result[14], result[15]);
            }
            catch (NoSuchAlgorithmException e)
            {
                e.printStackTrace();
            }
            catch (UnsupportedEncodingException e)
            {
                e.printStackTrace();
            }
            return null;
        }

        /**
         * parse the Soap response and build the manifest
         * @param xpp
         * @return
         * @throws XmlPullParserException
         * @throws IOException
         */
        public static GetTabletConfiguration createFromSoap(XmlPullParser xpp)
                throws XmlPullParserException, IOException {

            GetTabletConfiguration result = new GetTabletConfiguration();
            String mName = null;
            String mValue = null;

            for (int eventType = xpp.next(); eventType != XmlPullParser.END_DOCUMENT; eventType=xpp.next() ) {
                switch (eventType){
                    case XmlPullParser.START_DOCUMENT:
                        break;
                    case XmlPullParser.START_TAG: {
                        // filter out the soap envelope tags
                        if (SOAP_ENV.contains("_" + xpp.getName()))
                            break;
                        // have the enum do the conversion one time
                        // have to switch to uppercase because 'instance' is a java reserved word so it can't be
                        // the value of an enum
//                        Log.d(TAG, xpp.getName() );
                        switch (NODES.getTag("_" + xpp.getName())){
                            case _GetTabletConfigurationResponse:
                                break;
                            // straight name/value pairs
                            case _GMTOffset:
                            case _RouterSSID:
                            case _SiteChain:
                            case _SitePCIPAddress:
                            case _SpreadPort:
                            case _HostIPAddress:
                            case _HostName:
                            case _IsValid:
                            case _RouterPassword:
                                result.map.put(xpp.getName(), xpp.nextText() );
                                break;

                            case _SiteLocation:
                                break;
                            // for each of these set the name here, and the value from Name.
                            case _Address:
                                break;
                            case _City:
                            case _Country:
                            case _State:
                                mName = xpp.getName();
                                break;
                            case _StreetAddress1:
                                result.map.put("Address1", xpp.nextText());
                                break;
                            case _StreetAddress2:
                                result.map.put("Address2", xpp.nextText());
                                break;
                            case _ZipCode:
                                break;
                            case _ZipCodeValue:
                                result.map.put("Zip", xpp.nextText());
                                break;
                            case _Id:
                                result.map.put("SiteId", xpp.nextText());
                                break;
                            case _Name:
                                mValue = xpp.getText();
                                break;
                            case _SitePhone:
                                result.map.put("Phone", xpp.nextText());
                                break;
                            case _TabletConfig:
                                break;
                            case _KeyValuePairOfstringstring:
                                break;
                            case _key:
                                mName = xpp.nextText();
                                break;
                            case _value:
                                mValue = xpp.nextText();
                                break;
                            default:
                                if (xpp.getText()!=null) {
                                    result.map.put(xpp.getName(), xpp.nextText());
                                }
                        }
                    } break;

                    case XmlPullParser.END_TAG:
//                        Log.d(TAG, "/"+xpp.getName() );
                        if (SOAP_ENV.contains("_" + xpp.getName()))
                            break;

                        switch (NODES.getTag("_" + xpp.getName())){
                            case _GetTabletConfigurationResponse:
                                break;

                            case _SiteLocation:
                                result.map.put("Name", mValue);

                                break;
                            // name and value were set in the start tags for item and Name
                            case _Address:
                                break;
                            case _City:
                            case _Country:
                            case _State:
                                result.map.put(mName, mValue);
                                break;
                            case _KeyValuePairOfstringstring:
                                // name and value were set in the start tags for key/value
                                if (mName==null || mValue==null){
                                    //                                    Log.d(TAG, "[n:"+mName +" v:"+ mValue+"]");
                                } else {
                                    result.map.put(mName, mValue);
                                }
                                break;
                            default:
                        }
                        break;
                    case XmlPullParser.END_DOCUMENT:
                        break;
                    case XmlPullParser.TEXT:
                        break;
                    default:
                }
            }

//            Log.d(TAG, "result:"+result.map);
            return result;
        }

    }

}
